/*********************************************************************
 * Desktop Station one R1 Sketch for Mr Push R2
 *
 * Copyright (C) 2015 Yaasan
 *
 */

// include the library code:
#include "Wire.h"
#include <EEPROM.h>
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include "DSCoreM_Type.h"
#include "DSCoreM_Common.h"
#include "DSCoreM_List.h"
#include "DSCoreM_DCC.h"
#include "DSCoreM_MM2.h"
#include "DSCoreM.h"
#include "I2CLCDLib.h"
#include "Wire.h"
#include "Functions.h"
#include "NewSerial.h"
#include "TrackReporterS88_DS.h"


#define MAX_S88DECODER 1
#define RELPYERROR_300 "300 Command error"
#define RELPYERROR_301 "301 Syntax error"
#define RELPYERROR_302 "302 receive timeout"
#define RELPYERROR_303 "303 Unknown error"
#define RELPYERROR_NONE ""

NewSerial SerialDS;


/* Decralation classes */
DSCoreLib DSCore;
TrackReporterS88_DS reporter(MAX_S88DECODER);


/* LCD functions */
I2CLCDLib LCD;

/* Parser */
String request;
String function;
word arguments[4];
word numOfArguments;
boolean result;

/* Button */

uint8_t gLastButton = 1;


//Task Schedule
unsigned long gPreviousL5 = 0; //   50ms interval
unsigned long gPreviousL8 = 0; // 1000ms interval
unsigned long gPreviousL9 = 0; // 2000ms interval



void printString(const char *s, int x, int y);
void DisplayTopLCD(word inAddress);
void ReplyPowerPacket(byte inPower);
void TogglePower();

/***********************************************************************
* 
* Boot setup
* 
* 
************************************************************************/

void setup()
{
	DSCore.Init();

	//Pin mode
	pinMode(A3, INPUT);
	digitalWrite(A3, 1); //pullup
	pinMode(A0, OUTPUT);
	digitalWrite(A0, 0); //Reset High
	delay(50);
	digitalWrite(A0, 1); //Reset High
	delay(50);
	
	
	/* Clear data of the S88 decoder */
	reporter.refresh();
	
	/* LCD init */
	LCD.begin();

	LCD.lcd_setCursor(0, 0);
	LCD.lcd_printStr("DSone R2");
	LCD.lcd_setCursor(0, 1);
	LCD.lcd_printStr("20161107"); 
	delay(50);

	/* Debug message for PC */
	SerialDS.Init();
	SerialDS.println("--------------------------------------");
	SerialDS.println("Desktop Station one firmware r2       ");
	SerialDS.println("--------------------------------------");
	SerialDS.println("100 Ready");
	
	//Reset task
	gPreviousL5 = millis();
	gPreviousL8 = millis();
	
	
	sei();    //ENABLE INTERRUPTION
	
}






boolean parse() 
{

	String request = SerialDS.getRequest();

	char aTemp[48];
	request.toCharArray(aTemp, 30);

	int lpar = request.indexOf('(');
	if (lpar == -1) 
	{
		return false;
	}

	function = String(request.substring(0, lpar));
	function.trim();

	int offset = lpar + 1;
	int comma = request.indexOf(',', offset);
	numOfArguments = 0;
	
	while (comma != -1) 
	{
		String tmp = request.substring(offset, comma);
		tmp.trim();
		arguments[numOfArguments++] = stringToWord(tmp);
		offset = comma + 1;
		comma = request.indexOf(',', offset);
	}

	int rpar = request.indexOf(')', offset);
	while (rpar == -1) 
	{
		return false;
	}

	if (rpar > offset) 
	{
		String tmp = request.substring(offset, rpar);
		tmp.trim();
		arguments[numOfArguments++] = stringToWord(tmp);
	}

	return true;
}

void DisplayTopLCD(word inAddress)
{
	char aTempStr[16];
	byte aDcc = 0;
	word aProtocol = DSCore.GetLocIDProtocol(inAddress >> 8);
	
	switch(aProtocol)
	{
	case ADDR_DCC:
	case ADDR_ACC_DCC:
		aDcc = 1;
		break;
	}
	
	
	LCD.lcd_setCursor(0, 0);
	sprintf(aTempStr, "%s %4d%s", DSCore.IsPower() == true ? "ON" : "OF" , inAddress - aProtocol, aDcc == 1 ? "D" : "M");
	LCD.lcd_printStr(aTempStr); 
	
}

boolean dispatch() 
{
	byte aValue;
	byte aValueHigh;
	boolean aResult;
	char aTempStr[16];
	word aLocProtocol;
	byte aDir = 0;
	long aSpeed = 0;
	
	if ((function == "setLocoDirection") || ((function == "sLD")))
	{
		DisplayTopLCD(arguments[0]);
		
		LCD.lcd_setCursor(0, 1);
		sprintf(aTempStr, "DIR: %s", arguments[1] == 1 ? "FWD" : "REV");
		LCD.lcd_printStr(aTempStr); 
		
		if( arguments[1] > 0)
		{
			//FWD=1,REV=2を FWD=0,REV=1に変換
			aDir = arguments[1] - 1;
			
			//異常値はREV固定
			if(aDir > 1)
			{
				aDir = 1;
			}
		}
		else
		{
			//FWD固定
			aDir = 0;
		}
		
		return DSCore.SetLocoDirection(arguments[0], aDir);
	
	}
	else if ((function == "setLocoFunction") || ((function == "sLF")))
	{
		DisplayTopLCD(arguments[0]);
		LCD.lcd_setCursor(0, 1);
		sprintf(aTempStr, "F%02d %s ", arguments[1],  arguments[2] == 1 ? "ON " : "OFF");
		LCD.lcd_printStr(aTempStr); 
		
		return DSCore.SetLocoFunction(arguments[0], arguments[1], arguments[2]);
	
	}
	else if ((function == "setTurnout") || ((function == "sTO")))
	{
		DisplayTopLCD(arguments[0]);
		
		LCD.lcd_setCursor(0, 1);
		sprintf(aTempStr, "ACC %s", arguments[1] == 1 ? "Str|" : "Div/");
		LCD.lcd_printStr(aTempStr); 
		
		return DSCore.SetTurnout(arguments[0], (byte)arguments[1]);
	
	}
	else if ((function == "setPower") || ((function == "sPW")))
	{
		LCD.lcd_setCursor(0, 0);
		sprintf(aTempStr, "Power%s", arguments[0] == 1 ? "On " : "Off");
		LCD.lcd_printStr(aTempStr); 
		LCD.lcd_setCursor(0, 1);
		LCD.lcd_printStr("        "); 
		
		return DSCore.SetPower(arguments[0]);
	
	}
	else if ((function == "setLocoSpeed") || ((function == "sLS")))
	{
		
		aSpeed = (arguments[1] * 25) >> 8;
		
		DisplayTopLCD(arguments[0]);
		LCD.lcd_setCursor(0, 1);
		sprintf(aTempStr, "SPD %3d%c", aSpeed, 0x52);
		LCD.lcd_printStr(aTempStr); 
		
		return DSCore.SetLocoSpeedEx(arguments[0], arguments[1], 2);
	}
	else if ((function == "reset") || ((function == "sRS")))
	{
		SerialDS.println("100 Ready");
		return true;
	}
	/* reset */
	else if ((function == "setPing") || ((function == "sPI")))
	{
		/* Type(2bytes), Version(2bytes), UID(4bytes) */
		SerialDS.println("@DSG,00,03,00,02,00,00,00,00");
		return true;
	}
	else if ((function == "getLocoConfig") || ((function == "gCV")))
	{
		aValue = 0;
		
		LCD.lcd_setCursor(0, 0);
		sprintf(aTempStr, "CVNo.%3d", arguments[1]);
		LCD.lcd_printStr(aTempStr); 
		LCD.lcd_setCursor(0, 1);
		LCD.lcd_printStr("Reading."); 
		
		aResult = DSCore.ReadConfig( arguments[1], &aValue, 0);
		
		//Result
		if( aResult != true)
		{
			sprintf(aTempStr, "Read Err");
		}
		else
		{
			sprintf(aTempStr, "   =>%3d", aValue);
		}
		
		LCD.lcd_setCursor(0, 1);
		LCD.lcd_printStr(aTempStr); 

		SerialDS.print("@CV,");
		SerialDS.printDEC(arguments[0]);
		SerialDS.print(",");
		SerialDS.printDEC(arguments[1]);
		SerialDS.print(",");
		
		if( aResult == true)
		{
			SerialDS.printDEC(aValue);
		}
		else
		{
			SerialDS.printDEC(0xFFFF);
		}
		
		SerialDS.println(",");
		
		return true;
	}
	else if (function == "getS88")
	{
		int aMaxS88Num = MAX_S88DECODER;
		
		if( arguments[0] > 0)
		{
			aMaxS88Num = arguments[0];
		}
		
		//Send a S88 sensor reply 
		SerialDS.print("@S88,");

		reporter.refresh(aMaxS88Num);

		word aFlags = 0;

		for( int j = 0; j < aMaxS88Num; j++)
		{
		  aFlags = (reporter.getByte((j << 1) + 1) << 8) + reporter.getByte(j << 1);
		  
		  SerialDS.printHEX((word)aFlags);
		  SerialDS.print(",");
		}
			
		SerialDS.println("");
		
		return true;
	} /* getS88 */
	else if ((function == "setLocoConfig") || ((function == "sCV")))
	{
		LCD.lcd_setCursor(0, 0);
		sprintf(aTempStr, "CVNo.%3d", arguments[1]);
		LCD.lcd_printStr(aTempStr); 
		
		sprintf(aTempStr, "WRT<=%3d", arguments[2]);
		LCD.lcd_setCursor(0, 1);
		LCD.lcd_printStr(aTempStr); 

		
		return DSCore.WriteConfig_Dir( arguments[1], arguments[2]);
	}
	else
	{
		return false;
	}
}

void loop()
{
	while(1)
	{
		
		/* Reset watchdog timer. */
		//wdt_reset(); 
		
		//パルスジェネレータのスキャン
		DSCore.Scan();	

		
		char *aReplyText = RELPYERROR_NONE;
		
		if( (millis() - gPreviousL8) >= 100)
		{
			uint8_t aCurrentButton = digitalRead(A3);
			
			if( (gLastButton == 1) && ( aCurrentButton == 0))
			{
				/* Power toggle */
				
				TogglePower();
				
			}
			
			gLastButton = aCurrentButton;
			
			//Reset task
			gPreviousL8 = millis();
			
		}
		
		
		if( SerialDS.received())
		{
			
	  		if (parse()) {
				if (dispatch()) {
					aReplyText = "200 Ok";
				} else {
					//SerialDS.println(function);
					aReplyText = RELPYERROR_300;
				}
	  			
			} else {
				aReplyText = RELPYERROR_301;
			}
			
			/* Reply to Desktop Station */
			SerialDS.println(aReplyText);
			
			SerialDS.clearReceive();
		}
		else
		{
			/* Nothing to do */
			
		}
	}
}

void TogglePower()
{
	char aTempStr[16];
	uint8_t aPower = DSCore.IsPower();
	
	if( aPower == 0)
	{
		aPower = 1;
	}
	else
	{
		aPower = 0;
	}
	
	LCD.lcd_setCursor(0, 0);
	sprintf(aTempStr, "Power%s", aPower == 1 ? "On " : "Off");
	LCD.lcd_printStr(aTempStr); 
	LCD.lcd_setCursor(0, 1);
	LCD.lcd_printStr("        "); 
	
	DSCore.SetPower(aPower);
	ReplyPowerPacket(aPower);
}

void ReplyPowerPacket(byte inPower)
{
	/* Power */
	SerialDS.print("@PWR,");
	SerialDS.printHEX(inPower);
	SerialDS.println(",");
}


void printString(const char *s, int x, int y)
{
	LCD.lcd_setCursor(x, y);
	LCD.lcd_printStr(s);
}

void clearDisplay()
{
	LCD.lcd_clear();           //clear the screen and set start position to top left corner

}
